<?php
/**
 * @filename TokenInterface.php
 * @author Jerry Yan <792602257@qq.com>
 * @date 2020/12/17 14:23
 */


namespace JerryYan\DSL\Token;


/**
 * Token链表基础结构
 * @package JerryYan\DSL\Token
 * @author Jerry Yan <792602257@qq.com>
 * @date 2021/2/22 14:00
 */
abstract class TokenInterface
{
    /** @var ?TokenInterface 上一个Token */
    protected $prevToken;
    /** @var ?TokenInterface 下一个Token */
    protected $nextToken=NULL;
    /** @var string 原始数据 */
    protected $_raw = "";
    /** @var array 定义别名 */
    public static $alias = [];
    /** @var array 定义正则别名 */
    public static $regexAlias = [];

    public function __construct(string $original)
    {
        $this->_raw = $original;
    }

    /**
     * 设置链表上级
     * @param self|null $token
     * @author Jerry Yan <792602257@qq.com>
     * @date 2021/2/22 14:01
     */
    public function setPrevToken(?self $token): void
    {
        $this->prevToken = $token;
    }
    /**
     * 设置链表下级
     * @param self|null $token
     * @author Jerry Yan <792602257@qq.com>
     * @date 2021/2/22 14:02
     */
    public function setNextToken(?self $token): void
    {
        $this->nextToken = $token;
    }

    /**
     * 获取链表第一个元素
     * @return $this
     * @author Jerry Yan <792602257@qq.com>
     * @date 2021/1/22 10:01
     */
    public function getFirstToken(): TokenInterface
    {
        return $this->hasPrevToken() ? $this->getPrevToken()->getFirstToken() : $this;
    }
    /**
     * 获取链表最后一个元素
     * @return $this
     * @author Jerry Yan <792602257@qq.com>
     * @date 2021/2/19 11:05
     */
    public function getLastToken(): TokenInterface
    {
        return $this->hasNextToken() ? $this->getNextToken()->getLastToken() : $this;
    }
    public function hasPrevToken(): bool
    {
        return $this->prevToken !== NULL;
    }
    public function hasNextToken(): bool
    {
        return $this->nextToken !== NULL;
    }
    public function getPrevToken(): ?TokenInterface{
        return $this->prevToken;
    }
    public function getNextToken(): ?TokenInterface{
        return $this->nextToken;
    }
    public function getPrevTokenLength(): int
    {
        if ($this->hasPrevToken()) return $this->getPrevToken()->getPrevTokenLength() + 1;
        else return 0;
    }
    public function getNextTokenLength(): int
    {
        if ($this->hasNextToken()) return $this->getNextToken()->getNextTokenLength() + 1;
        else return 0;
    }
    public function getLength(): int
    {
        return $this->getPrevTokenLength() + $this->getNextTokenLength() + 1;
    }
    public function count(): int
    {
        return $this->getLength();
    }
    public function getRawString(): string
    {
        return $this->_raw;
    }
}